<?php

namespace Bydedi\Tracking;

use Query;
use Panier;
use Commande;
use Symfony\Component\HttpFoundation\Cookie;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class Tracking
 * @package Bydedi\Tracking
 */
class Tracking
{
    /**
     * @var array|mixed
     */
    static public $tracker = array();

    /**
     * @var Request
     */
    protected $request;

    public function __construct(Request $request)
    {
        $this->request = $request;

        $file = __DIR__ . '/../../config/tracking.php';
        if (file_exists($file)) {
            $this->tracker = require $file;
        }
    }

    public function get($action, $param = array(), &$smarty = "")
    {

        if (method_exists($this, $action)) {
            list($action, $param['extra']) = $this->$action($param);
        }

        $param['smarty']=&$smarty;
        foreach (self::$tracker as $class) {
            if (method_exists($class, $action)) {
                call_user_func_array(array($class, $action), array($this->request, $param));
            }
        }
    }

    public function user($param = array())
    {
        return array('user', $param);
    }

    public function product($param = array())
    {
        return array('product', $param);
    }

    public function paiementPassif($param = array())
    {

        // recuperation des données
        $commande = new Commande;
        list($arrCommande) = $commande->get(array('commande_id' => $param['commande_id']));
        $info['produits'] = $commande->getProduit(array('commande_id' => $param['commande_id']));

        // preparation des données
        $info['commande_id'] = 'C' . $arrCommande['commande_id'];
        $info['commande_stotal_ht'] = round($arrCommande['commande_stotal_ht'], 2);
        $info['commande_stotal_ttc'] = $arrCommande['commande_stotal_ttc'];
        $info['commande_stotal_taxe'] = round($arrCommande['commande_stotal_taxe'], 2);
        $info['commande_port_ht'] = round($arrCommande['commande_port_ht'], 2);
        $info['commande_port_ttc'] = $arrCommande['commande_port_ttc'];
        $info['commande_total_ht'] = round($arrCommande['commande_total_ht'], 2);
        $info['commande_total_ttc'] = $arrCommande['commande_total_ttc'];
        $info['commande_total_taxe'] = round($arrCommande['commande_total_taxe'], 2);
        $info['commande_referer'] = $arrCommande['commande_referer'];
        $info['livraison_ville'] = $arrCommande['livraison_ville'];
        $info['livraison_cp'] = $arrCommande['livraison_cp'];
        $info['livraison_pays'] = $arrCommande['livraison_pays'];
        $info['livraison_pays_code'] = $arrCommande['livraison_pays_code'];
        $info['paiement_nom_nom'] = $arrCommande['commande_paiement_nom'];

        // envoi des données
        return array('paiement', $info);
    }

    public function paiementActif($param = array())
    {

        // recuperation des donnes
        $panier = Panier::getInstance();
        $arrProduit = $panier->getProducts();

        list($sTotalHT, $sTotalTTC) = $panier->getTotalProducts();
        list($aPayerHT, $aPayerTTC) = $panier->getAPayer();
        $shipping = $panier->getShipping();

        $addrShipping = $panier->getAddressShipping();

        // preparation des données
        $info['commande_id'] = 'T' . $panier->getReference();
        $info['commande_stotal_ht'] = $sTotalHT;
        $info['commande_stotal_ttc'] = $sTotalTTC;
        $info['commande_stotal_taxe'] = $info['commande_stotal_ttc'] - $info['commande_stotal_ht'];
        $info['commande_port_ht'] = $shipping->port_ht;
        $info['commande_port_ttc'] = $shipping->port_ttc;
        $info['commande_total_ht'] = $aPayerHT;
        $info['commande_total_ttc'] = $aPayerTTC;
        $info['commande_total_taxe'] = $info['commande_total_ttc'] - $info['commande_total_ht'];
        $info['commande_referer'] = ''; // PENDING : lorsque l'application referer
        $info['livraison_ville'] = $addrShipping->adresse_ville;
        $info['livraison_cp'] = $addrShipping->adresse_cp;
        $info['livraison_pays'] = $addrShipping->pays_nom_nom;
        $info['livraison_pays_code'] = $addrShipping->pays_iso_code_2;

        $info['produits'] = array();
        foreach ($arrProduit as $product) {
            array_push(
                $info['produits'],
                array(
                    'produit_ref' => $product['produit_ref'],
                    'produit_nom' => $product['produit_nom'],
                    'prix_ttc' => $product['produit_prix_ttc'],
                    'qte' => $product['produit_quantite'],
                )
            );
        }

        // envoi des données
        return array('paiement', $info);

    }


    public function recapitulatif($param = array())
    {
        $panier = Panier::getInstance();
        $arrProduit = $panier->getProducts();

        list($sTotalHT, $sTotalTTC) = $panier->getTotalProducts();
        list($aPayerHT, $aPayerTTC) = $panier->getAPayer();
        $shipping = $panier->getShipping();

        $addrShipping = $panier->getAddressShipping();

        // preparation des données
        $info['commande_stotal_ht'] = $sTotalHT;
        $info['commande_stotal_ttc'] = $sTotalTTC;
        $info['commande_stotal_taxe'] = $info['commande_stotal_ttc'] - $info['commande_stotal_ht'];
        $info['commande_port_ht'] = $shipping->port_ht;
        $info['commande_port_ttc'] = $shipping->port_ttc;
        $info['commande_total_ht'] = $aPayerHT;
        $info['commande_total_ttc'] = $aPayerTTC;
        $info['commande_total_taxe'] = $info['commande_total_ttc'] - $info['commande_total_ht'];
        $info['commande_referer'] = ''; // PENDING : lorsque l'application referer
        $info['livraison_ville'] = $addrShipping->adresse_ville;
        $info['livraison_cp'] = $addrShipping->adresse_cp;
        $info['livraison_pays'] = $addrShipping->pays_nom_nom;
        $info['livraison_pays_code'] = $addrShipping->pays_iso_code_2;

        $info['produits'] = array();
        foreach ($arrProduit as $product) {
            array_push(
                $info['produits'],
                array(
                    'produit_ref' => $product['produit_ref'],
                    'produit_nom' => $product['produit_nom'],
                    'prix_ttc' => $product['produit_prix_ttc'],
                    'qte' => $product['produit_quantite'],
                )
            );
        }

        // envoi des données
        return array('recapitulatif', $info);
    }

    /**
     * @return bool|Cookie
     */
    public function initReferer()
    {
        $session = $this->request->getSession();

        $referer = $this->request->get("referer", $this->request->get("utm_source"));

        if (!empty($referer)) {
            if ($session->has('referer') == false || $session->get('referer') != $referer) {
                $session->set('referer', $referer);
            }

            $expire = time() + (3600 * 24 * 30); // valable 30 jours
            return new Cookie('referer', $referer, $expire);
        }

        return false;
    }

    /**
     * @param $type
     * @param $id
     * @param bool $delete
     *
     * @return bool|Cookie
     */
    public function setReferer($type, $id, $delete = false)
    {
        $session = $this->request->getSession();

        if ($session->has('referer')) {
            $query = new Query();
            switch ($type) {
                case 'commande':
                    $query->update('commande')
                        ->value('commande_referer', $session->get('referer'))
                        ->where('commande_id', $id);
                    \SiteSQL::getInstance()->query($query->build());
                    break;
            }
        }

        if ($delete == true) {
            if ($session->has('referer')) {
                $session->remove('referer');
            }

            $expire = time() - 3600;
            return new Cookie('referer', '', $expire);
        }

        return false;
    }
}
